#!/usr/bin/env perl
print "Content-Type: text/html; charset=utf-8\n\n";

use WebService::Solr;
use WebService::Solr::Query;
use WebService::Solr::Response;
use CGI;
use HTML::Entities;
use List::Util 'min';
use URI::Escape;
use Data::Dumper;
use Encode 'encode';

my $request = new CGI;
$request->charset('utf-8');

my $unique_key = "url";
my $default_start = 0;
my $default_rows = 10;
my @default_fq = ();
my $default_fl = "*,score";
my $default_facet = "true";
my @default_facet_field = ("genre");
my $default_facet_mincount = 1;
my $default_hl = "true";
my $default_hl_fl = "title,summary";
my $default_hl_snipets = 2;
my $default_hl_simple_pre = "<strong>";
my $default_hl_simple_post = "</strong>";

my $start = $request->param("start") ? $request->param("start") : $default_start;
my $rows = $request->param("rows") ? $request->param("rows") : $default_rows;
my $query = $request->param('q');
my @fq = $request->param("fq") ? $request->param("fq") : @default_fq;
my $fl = $request->param("fl") ? $request->param("fl") : $default_fl;
my $facet = $request->param("facet") ? $request->param("facet") : $default_facet;
my @facet_field = $request->param("facet.field") ? $request->param("facet.field") : @default_facet_field;
my $facet_mincount = $request->param("facet_mincount") ? $request->param("facet_mincount") : $default_facet_mincount;
my $hl = $request->param("hl") ? $request->param("hl") : $default_hl;
my $hl_fl = $request->param("hl_fl") ? $request->param("hl_fl") : $default_hl_fl;
my $hl_snipets = $request->param("hl_snipets") ? $request->param("hl_snipets") : $default_hl_snipets;
my $hl_simple_pre = $request->param("hl_simple_pre") ? $request->param("hl_simple_pre") : $default_hl_simple_pre;
my $hl_simple_post = $request->param("hl_simple_post") ? $request->param("hl_simple_post") : $default_hl_simple_post;
my $result = null;

my @tmp_fq = ();
foreach $q (@fq) {
	my @strlist = split(/:/, $q, 2);
	push(@tmp_fq, WebService::Solr::Query->new({$strlist[0] => $strlist[1]}));
}

my %params = (
	"start" => $start,
	"rows" => $rows,
	"fq" => \@tmp_fq,
	"fl" => $fl,
	"facet" => $facet,
	"facet.field" => \@facet_field,
	"facet.mincount" => $facet_mincount,
	"hl" => $hl,
	"hl.fl" => $hl_fl,
	"hl.simple.pre" => $hl_simple_pre,
	"hl.simple.post" => $hl_simple_post
);

if (not $query eq "") {
	$solr = WebService::Solr->new('http://localhost:8983/solr/solrbook');
	eval {
		$result = $solr->search(WebService::Solr::Query->new({-default => $query}), \%params);
	};
	if($@) {
		print "<html><head><title>SEARCH EXCEPTION</title><body><pre>" . $@ . "</pre></body></html>\n";
		exit(1)
	}
}

print "<html>\n";
print "	<head>\n";
print "		<link rel=\"stylesheet\" href=\"./search.css\" type=\"text/css\" />\n";
print "		<title>Perl Solr Client Example</title>\n";
print "	</head>\n";
print "	<body>\n";
print "		<div id=\"container\"><!-- start container -->\n";
print "			<div id=\"head\"><!-- start head -->\n";
print "				Perl Solr Client Example\n";
print "				<div id=\"searchbox\"><!-- start searchbox -->\n";
print "					<form	accept-charset=\"utf-8\" method=\"get\">\n";
print "						<label for=\"q\">Search:</label>\n";
print "						<input id=\"q\" name=\"q\" size=\"100\" type=\"text\" value=\"" . $request->escapeHTML($query) . "\"/>\n";
print "						<input id=\"start\" name=\"start\" type=\"hidden\" value=\"" . $request->escapeHTML($default_start) . "\"/>\n";
print "						<input id=\"rows\" name=\"rows\" type=\"hidden\" value=\"" . $request->escapeHTML($rows) . "\"/>\n";
print "						<input id=\"fl\" name=\"fl\" type=\"hidden\" value=\"" . $request->escapeHTML($fl) . "\"/>\n";
print "						<input id=\"facet\" name=\"facet\" type=\"hidden\" value=\"" . $request->escapeHTML($facet) . "\"/>\n";
for my $ff (@facet_field) {
	print "						<input id=\"facet_field\" name=\"facet.field\" type=\"hidden\" value=\"" . $request->escapeHTML($ff) . "\"/>\n";
}
print "						<input id=\"facet_mincount\" name=\"facet.mincount\" type=\"hidden\" value=\"" . $request->escapeHTML($facet_mincount) . "\"/>\n";
print "						<input id=\"hl\" name=\"hl\" type=\"hidden\" value=\"" . $request->escapeHTML($hl) . "\"/>\n";
print "						<input id=\"hl_fl\" name=\"hl.fl\" type=\"hidden\" value=\"" . $request->escapeHTML($hl_fl) . "\"/>\n";
print "						<input id=\"hl_simple_pre\" name=\"hl.simple.pre\" type=\"hidden\" value=\"" . $request->escapeHTML($hl_simple_pre) . "\"/>\n";
print "						<input id=\"hl_simple_post\" name=\"hl.simple.post\" type=\"hidden\" value=\"" . $request->escapeHTML($hl_simple_post) . "\"/>\n";
print "						<input type=\"submit\"/>\n";
print "					</form>\n";
print "				</div><!-- end searchbox -->\n";
print "			</div><!-- end head -->\n";

print "			<div id=\"main\"><!-- start main -->\n";
print "				<div id=\"searchresult\"><!-- start searchresult -->\n";


if ($result != null) {
	$total_num = $result->pager->total_entries;
	$disp_start_num = min ($start + 1, $total_num);
	$disp_end_num = min ($start+ $rows, $total_num);
	
	my %highlighting = $result->content->{ highlighting };
	print "					<div id=\"pageinfo\"><!-- start pageinfo -->\n";
	print "						Results " . $request->escapeHTML($disp_start_num) . " - " . $request->escapeHTML($disp_end_num) . " of " . $request->escapeHTML($total_num) . ":\n";
	print "					</div><!-- end pageinfo -->\n";
	for my $doc ($result->docs) {
		print "					<div id=\"document\"><!-- start document -->\n";
		my $key = "";
		for my $field ($doc->field_names) {
			if ($field eq $unique_key) {
				$key = $doc->value_for($field);
			}
		}
		
		for my $field ($doc->field_names) {
			print "						<div id=\"field\"><!-- start field -->\n";
			print "							<span id=\"fieldname\"><!-- start fieldname -->\n";
			print "								" . $request->escapeHTML($field) . ":&nbsp;\n";
			print "							</span><!-- end fieldname -->\n";
			print "							<span id=\"fieldvalue\"><!-- start fieldname -->\n";
			
			my $highlight_text = "";
			my $highlighting_ref = $result->content()->{"highlighting"}->{$key}->{$field};
			@highlighting = ();
			for (my $i = 0; $i < $hl_snipets; $i++) {
				my $h = $highlighting_ref->[$i];
				next if ( ! $h );
				push @highlighting, $h;
			}
			$highlight_text = join(" ... ", @highlighting);
			
			if ($highlight_text ne "") {
				print "								" . $highlight_text . "\n";
			} elsif ($doc->values_for($field) > 1) {
				print "								" . $request->escapeHTML(join(", ", $doc->values_for($field))) . "\n";
			} else {
				print "								" . $request->escapeHTML($doc->value_for($field)) . "\n";
			}
			print "							</span><!-- end fieldvalue -->\n";
			print "						</div><!-- end field -->\n";
		}
		print "					</div><!-- end document -->\n";
	}
	
	print "					<div id=\"pagination\"><!-- start pagenation -->\n";
	
	my $wsiz = 10;
	my $w1 = 5;
	my $w2 = 5;
	
	my $pcnt = int($total_num / $rows + (($total_num % $rows) == 0 ? 0 : 1));
	my $cpag = int($start / $rows + 1);
	my $wbgn = $cpag - $w1;
	my $wend = $cpag + $w2;
	
	if ($wbgn < 1) {
		$wbgn = 1;
		$wend = $wbgn + $wsiz;
		if ($wend > $pcnt + 1) {
			$wend = $pcnt + 1;
		}
	}
	if ($wend > $pcnt + 1) {
		$wend = $pcnt + 1;
		$wbgn = $wend - $wsiz;
		if ($wbgn < 1) {
			$wbgn = 1;
		}
	}
	
	if ($cpag > 1) {
		my $link = "./search.pl?";
		$link = $link . "q=" . uri_escape($query);
		for my $q (@fq) {
			$link = $link . "&fq=" . uri_escape($q);
		}
		$link = $link . "&start=" . uri_escape(($cpag - 2) * $rows);
		$link = $link . "&rows=" . uri_escape($rows);
		$link = $link . "&fl=" . uri_escape($fl);
		$link = $link . "&facet=" . uri_escape($facet);
		for my $ff (@facet_field) {
			$link = $link . "&facet.field=" . uri_escape($ff);
		}
		$link = $link . "&facet.mincount=" . uri_escape($facet_mincount);
		print "						<a href=\"" . $request->escapeHTML($link) . "\">&lt;&lt;</a>&nbsp;";
	}
	
	for (my $i = $wbgn; $i < $wend; $i++) {
		if ($cpag == $i) {
			print "						&nbsp;" + $request->escapeHTML($i) + "&nbsp;";
		} else {
			my $link = "./search.pl?";
			$link = $link . "q=" . uri_escape($query);
			for my $q (@fq) {
				$link = $link . "&fq=" . uri_escape($q);
			}
			$link = $link . "&start=" . uri_escape(($i - 1) * $rows);
			$link = $link . "&rows=" . uri_escape($rows);
			$link = $link . "&fl=" . uri_escape($fl);
			$link = $link . "&facet=" . uri_escape($facet);
			for my $ff (@facet_field) {
				$link = $link . "&facet.field=" . uri_escape($ff);
			}
			$link = $link . "&facet.mincount=" . uri_escape($facet_mincount);
			print "						&nbsp;<a href=\"" . $request->escapeHTML($link) . "\">" . $request->escapeHTML($i) . "</a>&nbsp;";
		}
	}
	
	if ($cpag < $pcnt) {
		my $link = "./search.pl?";
		$link = $link . "q=" . uri_escape($query);
		for my $q (@fq) {
			$link = $link . "&fq=" . uri_escape($q);
		}
		$link = $link . "&start=" . uri_escape($cpag * $rows);
		$link = $link . "&rows=" . uri_escape($rows);
		$link = $link . "&fl=" . uri_escape($fl);
		$link = $link . "&facet=" . uri_escape($facet);
		for my $ff (@facet_field) {
			$link = $link . "&facet.field=" . uri_escape($ff);
		}
		$link = $link . "&facet.mincount=" . uri_escape($facet_mincount);
		print "						&nbsp;<a href=\"" . $request->escapeHTML($link) . "\">&gt;&gt;</a>";
	}
	
	print "					</div><!-- end pagenation -->\n";
}

print "				</div><!-- end searchresult -->\n";
print "			</div><!-- end main -->\n";

if ($result != null) {
	if ($facet eq "true") {
		print "			<div id=\"left\"><!-- start left -->\n";
		for $ff (@facet_field) {
			print "				<div id=\"facet\"><!-- start facet -->\n";
			print "					" . $request->escapeHTML($ff);
			my $facet_ref = $result->facet_counts()->{"facet_fields"}->{$ff};
			my $i = 0;
			foreach ( @$facet_ref ) {
				my $k = $facet_ref->[$i]; $i++;
				my $v = $facet_ref->[$i]; $i++;
				next if ( ! $v );
				
				$link = "./search.pl?";
				$link = $link . "q=" . uri_escape($query);
				for my $q (@fq) {
					$link = $link . "&fq=" . uri_escape($q);
				}
				$link = $link . "&fq=" . uri_escape(encode('UTF-8',  $ff) . ":" . encode('UTF-8', $k));
				$link = $link . "&start=" . uri_escape($default_start);
				$link = $link . "&rows=" . uri_escape($rows);
				$link = $link . "&fl=" . uri_escape($fl);
				$link = $link . "&facet=" . uri_escape($facet);
				for $f (@facet_field) {
					$link = $link . "&facet.field=" . uri_escape($f);
				}
				$link = $link . "&facet.mincount=" . uri_escape(facet_mincount);
				print "					<div id=\"facet_value\"><!-- start facet_value -->\n";
				print "						<a href=\"" . $request->escapeHTML($link) . "\">" . $request->escapeHTML($k) . "</a>&nbsp;(" . $request->escapeHTML($v) . ")";
				print "					</div><!-- end facet_value -->\n";
			}
			print "				</div><!-- end facet -->\n";
		}
		print "			</div><!-- end left -->\n";
	}
}

print "			<div id=\"foot\"><!-- start foot -->\n";
print "				sample";
print "			</div><!-- end foot -->\n";
print "		</div><!-- end container -->\n";
print "	</body>\n";
print "</html>\n";
