#!/usr/bin/env ruby
print "Content-Type: text/html; charset=utf-8\n\n"

require 'rubygems'
require 'rsolr'
require "cgi-lib"
require "cgi"

request = CGI.new

unique_key = "url"
default_start = 0
default_rows = 10
default_fq = []
default_fl = "*,score"
default_facet = "true"
default_facet_field = ["genre"]
default_facet_mincount = 1
default_hl = "true"
default_hl_fl = "title,summary"
default_hl_snipets = 2
default_hl_simple_pre = "<strong>"
default_hl_simple_post = "</strong>"

start = request["start"] != "" ? request["start"].to_i : default_start
rows = request["rows"] != "" ? request["rows"].to_i : default_rows
query = request["q"]
fq = request["fq"] != "" ? request["fq"].to_a : default_fq
fl = request["fl"] != "" ? request["fl"] : default_fl
facet = request["facet"] != "" ? request["facet"] : default_facet
facet_field = request["facet.field"] != "" ? request["facet.field"].to_a : default_facet_field
facet_mincount = request["facet_mincount"] != "" ? request["facet_mincount"].to_i : default_facet_mincount
hl = request["hl"] != "" ? request["hl"] : default_hl
hl_fl = request["hl_fl"] != "" ? request["hl_fl"] : default_hl_fl
hl_snipets = request["hl_snipets"] != "" ? request["hl_snipets"].to_i : default_hl_snipets
hl_simple_pre = request["hl_simple_pre"] != "" ? request["hl_simple_pre"] : default_hl_simple_pre
hl_simple_post = request["hl_simple_post"] != "" ? request["hl_simple_post"] : default_hl_simple_post
result = nil

params = {
	"q" => query,
	"start" => start,
	"rows" => rows,
	"fq" => fq,
	"fl" => fl,
	"facet" => facet,
	"facet.field" => facet_field,
	"facet.mincount" => facet_mincount,
	"hl" => hl,
	"hl.fl" => hl_fl,
	"hl.simple.pre" => hl_simple_pre,
	"hl.simple.post" => hl_simple_post,
};

if query != "" then
	solr = RSolr.connect :url => 'http://localhost:8983/solr/solrbook'
	begin
		result = solr.get 'select', :params => params
	rescue => e
		print "<html><head><title>SEARCH EXCEPTION</title><body><pre>#{CGI.escapeHTML(e.message)}</pre></body></html>\n"
		exit 1
	end
end

print "<html>\n"
print "	<head>\n"
print "		<link rel=\"stylesheet\" href=\"./search.css\" type=\"text/css\" />\n"
print "		<title>Ruby Solr Client Example</title>\n"
print "	</head>\n"
print "	<body>\n"
print "		<div id=\"container\"><!-- start container -->\n"
print "			<div id=\"head\"><!-- start head -->\n"
print "				Ruby Solr Client Example\n"
print "				<div id=\"searchbox\"><!-- start searchbox -->\n"
print "					<form accept-charset=\"utf-8\" method=\"get\">\n"
print "						<label for=\"q\">Search:</label>\n"
print "						<input id=\"q\" name=\"q\" size=\"100\" type=\"text\" value=\"#{CGI.escapeHTML(query)}\"/>\n"
print "						<input id=\"start\" name=\"start\" type=\"hidden\" value=\"#{CGI.escapeHTML(default_start.to_s)}\"/>\n"
print "						<input id=\"rows\" name=\"rows\" type=\"hidden\" value=\"#{CGI.escapeHTML(rows.to_s)}\"/>\n"
print "						<input id=\"fl\" name=\"fl\" type=\"hidden\" value=\"#{CGI.escapeHTML(fl)}\"/>\n"
print "						<input id=\"facet\" name=\"facet\" type=\"hidden\" value=\"#{CGI.escapeHTML(facet)}\"/>\n"
facet_field.each{|ff|
	print "						<input id=\"facet_field\" name=\"facet.field\" type=\"hidden\" value=\"#{CGI.escapeHTML(ff)}\"/>\n"
}
print "						<input id=\"facet_mincount\" name=\"facet.mincount\" type=\"hidden\" value=\"#{CGI.escapeHTML(facet_mincount.to_s)}\"/>\n"
print "						<input id=\"hl\" name=\"hl\" type=\"hidden\" value=\"#{CGI.escapeHTML(hl)}\"/>\n"
print "						<input id=\"hl_fl\" name=\"hl.fl\" type=\"hidden\" value=\"#{CGI.escapeHTML(hl_fl)}\"/>\n"
print "						<input id=\"hl_simple_pre\" name=\"hl.simple.pre\" type=\"hidden\" value=\"#{CGI.escapeHTML(hl_simple_pre)}\"/>\n"
print "						<input id=\"hl_simple_post\" name=\"hl.simple.post\" type=\"hidden\" value=\"#{CGI.escapeHTML(hl_simple_post)}\"/>\n"
print "						<input type=\"submit\"/>\n"
print "					</form>\n"
print "				</div><!-- end searchbox -->\n"
print "			</div><!-- end head -->\n"

print "			<div id=\"main\"><!-- start main -->\n"
print "				<div id=\"searchresult\"><!-- start searchresult -->\n"

unless result.nil? then
	total_num = result["response"]["numFound"].to_i
	disp_start_num = [start + 1, total_num].min
	disp_end_num = [start + rows, total_num].min
	print "					<div id=\"pageinfo\"><!-- start pageinfo -->\n"
	print "						Results #{CGI.escapeHTML(disp_start_num.to_s)} - #{CGI.escapeHTML(disp_end_num.to_s)} of #{CGI.escapeHTML(total_num.to_s)}:\n"
	print "					</div><!-- end pageinfo -->\n"
	result["response"]["docs"].each {|doc|
		print "					<div id=\"document\"><!-- start document -->\n"
		key = ""
		doc.keys.sort.each {|field|
			if field == unique_key then
				key = doc[field]
			end
		}
		doc.keys.sort.each {|field|
			print "						<div id=\"field\"><!-- start field -->\n"
			print "							<span id=\"fieldname\"><!-- start fieldname -->\n"
			print "								#{CGI.escapeHTML(field)}:&nbsp;\n"
			print "							</span><!-- end fieldname -->\n"
			print "							<span id=\"fieldvalue\"><!-- start fieldname -->\n"
			
			highlight_text = ""
			if result["highlighting"].has_key?(key) && result["highlighting"][key].has_key?(field) then
				highlight_text = result["highlighting"][key][field].join(" ... ")
			end
			
			if highlight_text != "" then
				print "								#{highlight_text}\n"
			elsif doc[field].kind_of?(Array) then
				print "								#{CGI.escapeHTML(doc[field].join(", "))}\n"
			else
				print "								#{CGI.escapeHTML(doc[field].to_s)}\n"
			end
			print "							</span><!-- end fieldvalue -->\n"
			print "						</div><!-- end field -->\n"
		}
		print "					</div><!-- end document -->\n"
	}
	
	print "					<div id=\"pagination\"><!-- start pagenation -->\n"
	
	wsiz = 10
	w1 = 5
	w2 = 5
	
	pcnt = total_num / rows + ((total_num % rows) == 0 ? 0 : 1)
	cpag = start / rows + 1
	wbgn = cpag - w1
	wend = cpag + w2
	
	if wbgn < 1 then
		wbgn = 1
		wend = wbgn + wsiz
		if wend > pcnt + 1 then
			wend = pcnt + 1
		end
	end
	if wend > pcnt + 1 then
		wend = pcnt + 1
		wbgn = wend - wsiz
		if wbgn < 1 then
			wbgn = 1
		end
	end
	
	if cpag > 1 then
		link = "./search.rb?"
		link = link + "q=" + CGI.escape(query)
		fq.each {|q|
			link = link + "&fq=" + CGI.escape(q)
		}
		link = link + "&start=" + CGI.escape(((cpag - 2) * rows).to_s)
		link = link + "&rows=" + CGI.escape(rows.to_s)
		link = link + "&fl=" + CGI.escape(fl)
		link = link + "&facet=" + CGI.escape(facet)
		facet_field.each {|ff|
			link = link + "&facet.field=" + CGI.escape(ff)
		}
		link = link + "&facet.mincount=" + CGI.escape(facet_mincount.to_s)
		print "						<a href=\"#{link}\">&lt;&lt;</a>&nbsp"
	end
	
	for i in wbgn .. wend - 1 do
		if cpag == i then
			print "						&nbsp;#{CGI.escapeHTML(i.to_s)}&nbsp;"
		else
			link = "./search.rb?"
			link = link + "q=" + CGI.escape(query)
			fq.each {|q|
				link = link + "&fq=" + CGI.escape(q)
			}
			link = link + "&start=" + CGI.escape(((i - 1) * rows).to_s)
			link = link + "&rows=" + CGI.escape(rows.to_s)
			link = link + "&fl=" + CGI.escape(fl)
			link = link + "&facet=" + CGI.escape(facet)
			facet_field.each {|ff|
				link = link + "&facet.field=" + CGI.escape(ff)
			}
			link = link + "&facet.mincount=" + CGI.escape(facet_mincount.to_s)
			print "						&nbsp;<a href=\"#{link}\">#{CGI.escapeHTML(i.to_s)}</a>&nbsp;"
		end
	end
	
	if cpag < pcnt then
		link = "./search.rb?"
		link = link + "q=" + CGI.escape(query)
		fq.each {|q|
			link = link + "&fq=" + CGI.escape(q)
		}
		link = link + "&start=" + CGI.escape((cpag * rows).to_s)
		link = link + "&rows=" + CGI.escape(rows.to_s)
		link = link + "&fl=" + CGI.escape(fl)
		link = link + "&facet=" + CGI.escape(facet)
		facet_field.each {|ff|
			link = link + "&facet.field=" + CGI.escape(ff)
		}
		link = link + "&facet.mincount=" + CGI.escape(facet_mincount.to_s)
		print "						<a href=\"#{link}\">&gt;&gt;</a>&nbsp"
	end
	print "					</div><!-- end pagenation -->\n"
end
print "				</div><!-- end searchresult -->\n"
print "			</div><!-- end main -->\n"

unless result.nil? then
	if facet == "true" then
		print "			<div id=\"left\"><!-- start left -->\n"
		result["facet_counts"].keys.sort.each {|facet_type|
			result["facet_counts"][facet_type].keys.sort.each {|field|
				print "				<div id=\"facet\"><!-- start facet -->\n"
				print "					#{CGI.escapeHTML(field)}\n"
				i = 0
				while i < result["facet_counts"][facet_type][field].size do
					link = "./search.rb?"
					link = link + "q=" + CGI.escape(query)
					fq.each {|q|
						link = link + "&fq=" + CGI.escape(q)
					}
					if result["facet_counts"][facet_type][field][i].include?(" ") then
						link = link + "&fq=" + CGI.escape(field + ":\"" + result["facet_counts"][facet_type][field][i] + "\"")
					else
						link = link + "&fq=" + CGI.escape(field + ":" + result["facet_counts"][facet_type][field][i])
					end
					link = link + "&start=" + CGI.escape(default_start.to_s)
					link = link + "&rows=" + CGI.escape(rows.to_s)
					link = link + "&fl=" + CGI.escape(fl)
					link = link + "&facet=" + CGI.escape(facet)
					facet_field.each {|ff|
						link = link + "&facet.field=" + CGI.escape(ff)
					}
					link = link + "&facet.mincount=" + CGI.escape(facet_mincount.to_s)
					print "					<div id=\"facet_value\"><!-- start facet_value -->\n"
					print "						<a href=\"#{link}\">#{CGI.escapeHTML(result["facet_counts"][facet_type][field][i])}</a>&nbsp;(#{CGI.escapeHTML(result["facet_counts"][facet_type][field][i + 1].to_s)})\n"
					print "					</div><!-- end facet_value -->\n"
					i += 2
				end
				print "				</div><!-- end facet -->\n"
			}
		}
		print "			</div><!-- end left -->\n"
	end
end

print "			<div id=\"foot\"><!-- start foot -->\n"
print "				sample"
print "			</div><!-- end foot -->\n"
print "		</div><!-- end container -->\n"
print "	</body>\n"
print "</html>\n"
